<li class="aldryn-forms-button">
	<input id="aldryn-forms-field-{{ instance.pk }}" {% if instance.custom_classes %}class="{{ instance.custom_classes}}"{% endif %} type="submit" name="submit" value="{{ instance.label }}"/>
</li>