{% extends 'aldryn_forms/base_form.html' %}
{% load cms_tags %}

{% block form_elements %}
<ul>
    {% for error in form.non_field_errors %}
        <li class="aldryn-forms-error aldryn-forms-global-error">
            {{ error }}
        </li>
    {% endfor %}
    {% for plugin in instance.child_plugin_instances %}
        {% render_plugin plugin %}
    {% endfor %}
</ul>
{% endblock form_elements %}
